from .core import get_title_from_url, normalize_url


def get(url: str, markdown: bool = False) -> str:
    """
    Get title from URL.
    """
    normalized_url = normalize_url(url)
    return get_title_from_url(normalized_url, markdown)



from __future__ import annotations

import platform
import sys
from itertools import filterfalse
from pathlib import Path

from readchar import key, readchar

from . import __version__, core
from .util import console, includes

HELP_MESSAGE = """\
Command line tool to Get pageTitle From Url.

[bold]Usage:[/bold]
  gtfu https://example.com/
  gtfu https://example.com/ -m
  gtfu
  gtfu -h
  gtfu -V

[bold]Options:[/bold]
  [blue]-m, --markdown[/blue]  The page title will be copied to the clipboard in markdown format.

[bold]Global options:[/bold]
  [blue]-h, --help[/blue]      Show this help message and exit.
  [blue]-V, --version[/blue]   Show program's version number and exit.

See https://github.com/seijinrosen/gtfu for more information.\
"""
EXIT_MESSAGE = "\nBye."
PROMPT_MESSAGE = "Ctrl+C or Ctrl+D to abort."
PROMPT_URL_MESSAGE = "Enter URL: "
PROMPT_IS_MARKDOWN_MESSAGE = "In markdown? (y/N): "

HELP_FLAGS = {"-h", "--help", "help"}
VERSION_FLAGS = {"-V", "--version", "version"}
MARKDOWN_FLAGS = {"-m", "--markdown"}


def print_help_message() -> None:
    console.print(HELP_MESSAGE)


def print_version() -> None:
    console.print("gtfu:  ", __version__)
    console.print("Python:", platform.python_version())
    console.print("from:  ", Path(__file__).parent)


def prompt(is_markdown: bool) -> tuple[str, bool]:
    console.print(PROMPT_MESSAGE)
    try:
        user_input_url = prompt_url()
        is_markdown = prompt_is_markdown() if not is_markdown else True
    except (KeyboardInterrupt, EOFError):
        print(EXIT_MESSAGE)
        sys.exit()
    return user_input_url, is_markdown


def prompt_url() -> str:
    url = ""
    while url == "":
        console.print(PROMPT_URL_MESSAGE, end="", style="bold blue")
        url = console.input().strip()
    return url


def prompt_is_markdown() -> bool:
    console.print(PROMPT_IS_MARKDOWN_MESSAGE, end="", style="bold blue")
    # Match statements require Python 3.10 or newer
    ch = readchar()
    if ch == key.CTRL_C:
        raise KeyboardInterrupt
    if ch == key.CTRL_D:
        raise EOFError
    is_y = ch == "y"
    # match readchar():
    #     case key.CTRL_C:
    #         raise KeyboardInterrupt
    #     case key.CTRL_D:
    #         raise EOFError
    #     case ch:
    #         is_y = ch == "y"
    console.print("Yes" if is_y else "No")
    return is_y


def main(args: list[str]) -> None:
    if includes(args, HELP_FLAGS):
        print_help_message()
        return
    if includes(args, VERSION_FLAGS):
        print_version()
        return

    user_input_url = next(filterfalse(lambda x: x.startswith("-"), args), "")
    is_markdown = includes(args, MARKDOWN_FLAGS)
    if user_input_url == "":
        user_input_url, is_markdown = prompt(is_markdown)

    core.main(user_input_url, is_markdown)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gtfu']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.10.0,<5.0.0',
 'lxml>=4.8.0,<5.0.0',
 'pyperclip>=1.8.2,<2.0.0',
 'readchar>=3.0.5,<5.0.0',
 'requests>=2.27.1,<3.0.0',
 'rich>=12.0.1,<13.0.0']

entry_points = \
{'console_scripts': ['gtfu = gtfu.__main__:main']}

setup_kwargs = {
    'name': 'gtfu',
    'version': '1.3.0',
    'description': 'Command line tool to Get pageTitle From Url.',
    'long_description': '# gtfu\n\n[日本語](https://github.com/seijinrosen/gtfu/blob/main/README.ja.md) |\n[English](https://github.com/seijinrosen/gtfu/blob/main/README.md)\n\n[![PyPI](https://img.shields.io/pypi/v/gtfu)](https://pypi.python.org/pypi/gtfu)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/gtfu)](https://pypi.python.org/pypi/gtfu)\n[![Tests](https://github.com/seijinrosen/gtfu/actions/workflows/tests.yml/badge.svg)](https://github.com/seijinrosen/gtfu/actions/workflows/tests.yml)\n[![CodeQL](https://github.com/seijinrosen/gtfu/actions/workflows/codeql-analysis.yml/badge.svg)](https://github.com/seijinrosen/gtfu/actions/workflows/codeql-analysis.yml)\n[![codecov](https://codecov.io/gh/seijinrosen/gtfu/branch/main/graph/badge.svg)](https://codecov.io/gh/seijinrosen/gtfu)\n[![Downloads](https://pepy.tech/badge/gtfu)](https://pepy.tech/project/gtfu)\n[![Downloads](https://pepy.tech/badge/gtfu/month)](https://pepy.tech/project/gtfu)\n[![Downloads](https://pepy.tech/badge/gtfu/week)](https://pepy.tech/project/gtfu)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\nCommand line tool to Get pageTitle From Url.\n\n## Installation\n\nIt supports Python 3.7+.\n\n```sh\npip install gtfu\n```\n\n## Usage\n\nAfter installation, type the following command into your terminal application.\n\n### Standard mode\n\n```sh\ngtfu https://example.com/\n```\n\nThe page title will be copied to the clipboard.\n\n- `Example Domain`\n\n### Markdown mode\n\n```sh\ngtfu -m https://example.com/\n```\n\nThe page title will be copied to the clipboard in markdown format.\n\n- `[Example Domain](https://example.com/)`\n\n### Prompt mode\n\n```sh\ngtfu\n```\n\nAn interactive prompt will begin. You will be asked for the URL to be retrieved and whether you want it in Markdown format.\n',
    'author': 'seijinrosen',
    'author_email': '86702775+seijinrosen@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/seijinrosen',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
